if not LibStub then error("LibRaidEquip requires LibStub.") end
local MAJOR, MINOR = "LibRaidEquip-1.0", "$Revision: 1 $"
local lib = LibStub:NewLibrary(MAJOR, MINOR)

if not lib then return end

-- This library keeps track of a raid or group's equipment.
-- Usage:

local gear = {}
local queue = {}
local enabled = false
local inspectiontarget = nil

local frame = CreateFrame("Frame", nil, UIParent)

local function InspectIsOK()
	return not (InspectFrame and InspectFrame:IsShown())
end

local function print(msg)
    DEFAULT_CHAT_FRAME:AddMessage(msg, 1, 1, 1);
end

local function Inspect(guid)
	local _, _, _, _, _, unit, realm = GetPlayerInfoByGUID(guid)
	local g = {
		['slots'] = {},
		['ts'] = time()
	}
	local found = false
	for i = 1, 19 do
		local link = GetInventoryItemLink(unit, i)
		g.slots[i] = link
		if link then
			found = true
		end
	end
	if found then
		gear[guid] = g
		--print("done with gear for "..unit)
		
		queue[guid] = nil
	else
		print('failed inspecting '..unit)
	end
end

function frame:INSPECT_READY(guid)
	if queue[guid] then
		Inspect(guid)
		--ClearInspectPlayer()
	end
end

local timer = 0
function frame:OnUpdate(elapsed)
	timer = timer + elapsed
	if timer > 2 then
		-- A suitable interval has passed. Do some inspection.
		local count = 0
		for guid,i in pairs(queue) do
			local _, _, _, _, _, unit, realm = GetPlayerInfoByGUID(guid)
			count = count + 1
			if InspectIsOK() and CanInspect(unit) and CheckInteractDistance(unit, 1) and not UnitIsDead('player') then
				--print('time to inspect ' .. UnitName(unit)..', '..unit)
				NotifyInspect(unit)
				break
			end
		end
		if count == 0 then
			--print('all done!')
			frame:SetScript ("OnUpdate", nil)
		end
		timer = 0
	end
end

function frame:OnEvent(event, ...)
	if self[event] then
		self[event] (self, ...)
	end
end

local function IsInQueueOrCache(guid)
	return queue[guid] or gear[guid]
end

local function CheckIfUpdateNeeded()
	local needed = false
	if IsInRaid() then
		--print("raid")
		for i = 1, GetNumGroupMembers() do
			if not IsInQueueOrCache(UnitGUID('raid'..i)) then
				--print('Adding ' .. UnitName('raid'..i) .. ' to queue.')
				queue[UnitGUID('raid'..i)] = true
				needed = true
			end
		end
	elseif IsInGroup() then
		--print("group")
		for i = 1, GetNumGroupMembers() -1 do
			if not IsInQueueOrCache(UnitGUID('party'..i)) then
				--print('Adding ' .. UnitName('party'..i) .. ' to queue.')
				queue[UnitGUID('party'..i)] = true
				needed = true
			end
		end
	end
	return needed
end

function frame:GROUP_ROSTER_UPDATE()
	if CheckIfUpdateNeeded() then
		frame:SetScript ("OnUpdate", frame.OnUpdate)
	end
end

lib.HasGear = function(self, guid)
	if gear[guid] then
		return true
	else
		return false
	end
end

lib.GetGear = function(self, guid)
	if gear[guid] then
		return gear[guid].slots
	end
end

lib.Enable = function()
	if not enabled then
		enabled = true
		frame:SetScript ("OnEvent", frame.OnEvent)
		frame:RegisterEvent("GROUP_ROSTER_UPDATE")
		frame:RegisterEvent('INSPECT_READY')
		-- Debug - add player to queue
		--Inspect(UnitGUID("player"))
		print('LibRaidEquip enabled')
	end
end

lib.Disable = function()
	if enabled then
		enabled = false
		frame:SetScript ("OnEvent", nil)
		frame:SetScript ("OnUpdate", nil)
		frame:UnregisterEvent("GROUP_ROSTER_UPDATE")
		frame:UnregisterEvent('INSPECT_READY')
	end
end